from typing import Any


def format_doc(obj: Any = None, **kwargs: Any) -> Any:
    """Format the docstr"""
    if obj is None:
        return lambda o: format_doc(o, **kwargs)

    obj.__doc__ = obj.__doc__.format(**kwargs)
    return obj



from datetime import datetime
from benchwork import BenchSet, BenchSetTable

from .cases import (
    BenchCasePackageInfo,
    BenchCaseCreatingDict,
    BenchCaseCreatingDictWithPresevedKeys,
    BenchCaseCreatingDictWithMagicKeys,
    BenchCaseAccessValue,
    BenchCaseRecursiveDotAccess,
    BenchCaseAutomaticHierarchy,
    BenchCasePreservedKeys,
    BenchCaseDashKeys,
    BenchCaseFrozenDict,
    BenchCaseKeyTransform,
)
from .apis import APIBase
from .utils import format_doc


@format_doc(now=datetime.now())
class BenchSetPackageInfo(BenchSet):
    """As of {now}"""

    case = BenchCasePackageInfo
    api_base = APIBase
    title = "Package Information"

    def run_cases(self):
        out = [
            f"|Package|Version|Last Commit|Stars|Forks|Description|",
            f"|-------|-------|-----------|-----|-----|-----------|",
        ]
        for case in self.cases:
            ret = str(case.run())
            ret = ret.replace("\n", "<br />")
            out.append(ret)
        return "\n".join(out)


class BenchSetCreatingDict(BenchSetTable):
    """How the packages create an object of the dict subclass"""

    api_base = APIBase
    case = BenchCaseCreatingDict
    title = "Creating an object of the dict subclass"
    header = "Created"


class BenchSetCreatingDictWithPresevedKeys(BenchSetTable):
    """How the packages create a dict with preserved keys
    (e.g. `keys`, `values`, `items`, etc)

    Literally, `{"keys": 1}`
    """

    api_base = APIBase
    case = BenchCaseCreatingDictWithPresevedKeys
    title = "Creating a dict with preserved keys"
    header = "Created or error"


class BenchSetCreatingDictWithMagicKeys(BenchSetTable):
    """How the packages create a dict with magic keys
    (e.g. `__name__`, `__class__`, etc)

    Literally, `{"__name__": 1}`
    """

    api_base = APIBase
    case = BenchCaseCreatingDictWithMagicKeys
    title = "Creating a dict with magic keys"
    header = "Created or error"


class BenchSetAccessValue(BenchSetTable):
    """How the packages to access values

    Literally `1` from `{"a": {"b": {"c": 1}}}`
    """

    api_base = APIBase
    case = BenchCaseAccessValue
    title = "Accessing values"
    header = "Way to access value"


class BenchSetRecursiveDotAccess(BenchSetTable):
    """Whether recursive dot access is supported when there are lists
    in the dict

    Literally `<dict>.a.b[0].c` from `{"a": {"b": [{"c": 1}, {"d": 2}]}}`
    """

    api_base = APIBase
    case = BenchCaseRecursiveDotAccess
    title = "Recursive Dot Access"
    header = "Value or error"


class BenchSetAutomaticHierarchy(BenchSetTable):
    """Whether a hierarchical structure is created by dot notation

    Literally `<dict>.a.b.c = 1` creates `{"a": {"b": {"c": 1}}}`
    """

    api_base = APIBase
    case = BenchCaseAutomaticHierarchy
    title = "Automatic Hierarchy"
    header = "Created or error"


class BenchSetConflictKeys(BenchSet):
    """How to access values with conflict keys

    Literally, accessing values from `{"keys": 1, "__name__": 2}`
    """

    api_base = APIBase
    case = BenchCasePreservedKeys
    title = "Accessing values with preserved keys"

    def run_cases(self):
        out = [
            "|Package|`obj.keys`|`obj['keys']`"
            "|`obj.__name__`|`obj['__name__']`|",
            f"|---|---|---|---|---|",
        ]
        for case in self.cases:
            ret = case.run()
            ret = [str(r).replace("\n", "<br />") for r in ret]
            ret = [f"`{r}`" for r in ret]
            out.append(f"|{case.api.name}|{'|'.join(ret)}|")
        return "\n".join(out)


class BenchSetDashKeys(BenchSet):
    """How the values with keys with dash are accessed

    Literally `<dict>.a_b` for `{"a-b": 1}`
    """

    api_base = APIBase
    case = BenchCaseDashKeys
    title = "Accessing dashed keys"

    def run_cases(self):
        out = [
            "|Package|`obj.a_b`|`obj['a_b']`|`obj['a-b']`|",
            f"|---|---|---|---|",
        ]
        for case in self.cases:
            ret = case.run()
            ret = [str(r).replace("\n", "<br />") for r in ret]
            ret = [f"`{r}`" for r in ret]
            out.append(f"|{case.api.name}|{'|'.join(ret)}|")
        return "\n".join(out)


class BenchSetFrozenDict(BenchSetTable):
    """Whether the packages support frozen dicts"""

    api_base = APIBase
    case = BenchCaseFrozenDict
    title = "Frozen Dict Support"
    header = "Support? and how?"


class BenchSetKeyTransform(BenchSetTable):
    """Whether the packages support key transformation for dot access

    For example: making `<dict>.a_b` to access value from `{"a.b": 1}`
    """

    api_base = APIBase
    case = BenchCaseKeyTransform
    title = "Key transformation support"
    header = "Support? and how?"



from pyparam import Params
from benchwork import BenchSuite, run_suite

from .sets import (
    BenchSetPackageInfo,
    BenchSetCreatingDict,
    BenchSetCreatingDictWithPresevedKeys,
    BenchSetCreatingDictWithMagicKeys,
    BenchSetAccessValue,
    BenchSetRecursiveDotAccess,
    BenchSetAutomaticHierarchy,
    BenchSetConflictKeys,
    BenchSetDashKeys,
    BenchSetFrozenDict,
    BenchSetKeyTransform,
)


class BenchSuite(BenchSuite):
    """Benchmarking for dot-accessible dict packages in python

    [![deps](https://img.shields.io/librariesio/release/pypi/dotdict-bench?style=flat-square)](https://libraries.io/github/pwwang/dotdict-bench#repository_dependencies)

    More test ideas? [Submit an issue](https://github.com/pwwang/dotdict-bench/issues)!
    """
    set_classes = [
        BenchSetPackageInfo,
        BenchSetCreatingDict,
        BenchSetCreatingDictWithPresevedKeys,
        BenchSetCreatingDictWithMagicKeys,
        BenchSetAccessValue,
        BenchSetRecursiveDotAccess,
        BenchSetAutomaticHierarchy,
        BenchSetConflictKeys,
        BenchSetDashKeys,
        BenchSetFrozenDict,
        BenchSetKeyTransform,
    ]


def main():
    params = Params(
        help_on_void=False,
        prog="dotdict-bench",
        desc="Benchmarking for dot-accessible dict packages in python",
    )
    params.add_param(
        "token",
        default=None,
        type=str,
        desc="Github token or a file containing the token",
    )
    params.add_param(
        "outfile",
        default="<stdout>",
        desc="Output file",
    )
    args = params.parse()
    run_suite(BenchSuite, args, "dotdict-bench", args.outfile)

if __name__ == "__main__":
    main()



from __future__ import annotations
from ast import Dict
from gc import freeze

import json
from abc import ABC, abstractproperty
from os import access
from pathlib import Path
from tempfile import gettempdir
from typing import TYPE_CHECKING, Mapping

import addict
import box
import dotmap
import dotwiz
import easydict
import dotsi
import dictlib
import diot

import importlib_metadata as im
from github import Github
from benchwork import BenchAPI

if TYPE_CHECKING:
    from types import ModuleType


class APIBase(BenchAPI, ABC):

    _SUBCLASSES = None

    @abstractproperty
    def package(self) -> ModuleType:
        ...

    @abstractproperty
    def repo(self) -> str:
        ...

    @property
    def github_meta(self) -> Mapping[str, str]:
        cachefile = Path(gettempdir()).joinpath(
            f"dotdict-{self._name}-v{self.version}.github_meta.json"
        )
        if cachefile.is_file():
            return json.loads(cachefile.read_text())

        token = self.args.token
        if token and Path(token).is_file():
            token = Path(token).read_text().strip()

        g = Github(token, per_page=1)
        r = g.get_repo(self.repo)
        data = {
            "stars": r.stargazers_count,
            "forks": r.forks_count,
            "last_commit": r.get_commits()[0].last_modified,
            "descr": r.description,
        }
        cachefile.write_text(json.dumps(data))
        return data

    @property
    def _name(self) -> str:
        return self.package.__name__

    @property
    def name(self) -> str:
        return (
            f'<a target="_blank" href="https://github.com/{self.repo}">'
            f'{self._name}</a>'
        )

    @property
    def access_way(self) -> str:
        return "`<dict>.a.b.c` or `<dict>['a']['b']['c']`"

    @property
    def version(self) -> str:
        return im.version(self._name)

    @property
    def freeze(self) -> str:
        return "Not supported"

    @property
    def key_transform(self) -> str:
        return "Not supported"

    def create(self, data: Dict) -> Dict:
        return self.package.Dict(data)


class AddictAPI(APIBase):

    package = addict
    repo = "mewwts/addict"
    freeze = "`<dict>.freeze()/.unfreeze()`"


class BoxAPI(APIBase):

    package = box
    repo = "cdgriffith/Box"
    freeze = "`Box(<dict>, frozen_box=True)`"
    key_transform = "Using Conversion Box or Camel Killer Box"

    @property
    def _name(self) -> str:
        return "python-box"

    def create(self, data: Dict) -> Dict:
        return self.package.Box(data)


class DotMapAPI(APIBase):

    package = dotmap
    repo = "drgrib/dotmap"

    def create(self, data: Dict) -> Dict:
        return self.package.DotMap(data)


class DotwizAPI(APIBase):

    package = dotwiz
    repo = "rnag/dotwiz"
    key_transform = (
        "`DotWizPlus` turns special-cased keys, such as names with spaces, "
        "into valid snake_case words"
    )

    def create(self, data: Dict) -> Dict:
        return self.package.DotWiz(data)


class EasydictAPI(APIBase):

    package = easydict
    repo = "makinacorpus/easydict"

    def create(self, data: Dict) -> Dict:
        return self.package.EasyDict(data)


class DotsiAPI(APIBase):

    package = dotsi
    repo = "polydojo/dotsi"


class DictLibAPI(APIBase):

    package = dictlib
    repo = "srevenant/dictlib"
    access_way = (
        "`dictlib.dig(<dict>, 'a.b.c')` "
        "or `dictlib.Dict(<dict>).a.b.c`"
    )


class DiotAPI(APIBase):

    package = diot
    repo = "pwwang/diot"
    freeze = "`FrozenDiot(<dict>)` or `Diot(<dict>, diot_frozen=True)`"
    key_transform = (
        "Support custom transform function: `Diot(..., diot_transform =...)`"
    )

    def create(self, data: Dict) -> Dict:
        return self.package.Diot(data)



from benchwork import BenchCase


class BenchCasePackageInfo(BenchCase):

    def run(self):
        packageinfo = self.api.github_meta
        version = self.api.version

        return (
            f"|{self.api.name}"
            f"|{version}"
            f"|{packageinfo['last_commit']}"
            f"|{packageinfo['stars']}"
            f"|{packageinfo['forks']}"
            f"|[(i)](# \"{packageinfo['descr']}\")|"
        )


class BenchCaseCreatingDict(BenchCase):

    def run(self):
        try:
            out = self.api.create({"a": {"b": {"c": 1}}})
            return f"`{out}` **Type**: `{type(out).__name__}`"
        except KeyError as e1:
            return f"KeyError: {e1}"
        except AttributeError as e2:
            return f"AttributeError: {e2}"
        except Exception as e3:
            return e3


class BenchCaseCreatingDictWithPresevedKeys(BenchCase):

    def run(self):
        try:
            return self.api.create({"keys": 1})
        except KeyError as e1:
            return f"KeyError: {e1}"
        except AttributeError as e2:
            return f"AttributeError: {e2}"
        except Exception as e3:
            return e3


class BenchCaseCreatingDictWithMagicKeys(BenchCase):

    def run(self):
        try:
            out = self.api.create({"__name__": 1})
            return f"`{out}`"
        except KeyError as e1:
            return f"KeyError: {e1}"
        except AttributeError as e2:
            return f"AttributeError: {e2}"
        except Exception as e3:
            return e3


class BenchCaseAccessValue(BenchCase):

    def run(self):
        return self.api.access_way


class BenchCaseRecursiveDotAccess(BenchCase):

    def run(self):
        d = self.api.create({"a": {"b": [{"c": 1}, {"d": 2}]}})
        try:
            o = d.a.b[0].c
        except KeyError as e1:
            return f"KeyError: {e1}"
        except AttributeError as e2:
            return f"AttributeError: {e2}"
        else:
            return f"`{o}`"


class BenchCaseAutomaticHierarchy(BenchCase):

    def run(self):
        try:
            d = self.api.create({})
            d.a.b.c = 1
        except KeyError as e1:
            return f"KeyError: {e1}"
        except AttributeError as e2:
            return f"AttributeError: {e2}"
        else:
            return f"`{d}`"


class BenchCasePreservedKeys(BenchCase):

    def run(self):
        out = []
        try:
            d = self.api.create({"keys": 1, "__name__": 2})
        except Exception:
            return ["Can't create"] * 4

        try:
            o = d.keys
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        try:
            o = d["keys"]
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        try:
            o = d.__name__
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        try:
            o = d["__name__"]
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        return out


class BenchCaseDashKeys(BenchCase):

    def run(self):
        out = []
        try:
            d = self.api.create({"a-b": 1})
        except Exception:
            return ["Can't create"] * 3

        try:
            o = d.a_b
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        try:
            o = d["a_b"]
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        try:
            o = d["a-b"]
        except KeyError as e1:
            o = f"KeyError: {e1}"
        except AttributeError as e2:
            o = f"AttributeError: {e2}"
        except Exception as e3:
            o = str(e3)
        out.append(o)

        return out


class BenchCaseFrozenDict(BenchCase):

    def run(self):
        return self.api.freeze


class BenchCaseKeyTransform(BenchCase):

    def run(self):
        return self.api.key_transform



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dotdict_bench']

package_data = \
{'': ['*']}

install_requires = \
['PyGithub>=1.55,<2.0',
 'addict>=2.4.0,<3.0.0',
 'benchwork>=0.0.1,<0.0.2',
 'dictlib>=1.1.5,<2.0.0',
 'diot>=0.1.6,<0.2.0',
 'dotmap>=1.3.30,<2.0.0',
 'dotsi>=0.0.3,<0.0.4',
 'dotwiz>=0.4.0,<0.5.0',
 'easydict>=1.9,<2.0',
 'importlib-metadata>=4.12.0,<5.0.0',
 'pyparam>=0.5.3,<0.6.0',
 'python-box>=6.0.2,<7.0.0']

setup_kwargs = {
    'name': 'dotdict-bench',
    'version': '0.0.0',
    'description': 'Benchmarking for dot-accessible dict packages in python',
    'long_description': '# dotdict-bench\n\nBenchmarking for dot-accessible dict packages in python\n\n[![deps](https://img.shields.io/librariesio/release/pypi/dotdict-bench?style=flat-square)](https://libraries.io/github/pwwang/dotdict-bench#repository_dependencies)\n\nMore test ideas? [Submit an issue](https://github.com/pwwang/dotdict-bench/issues)!\n\n\n## Package Information\n\nAs of 2022-09-21 23:11:19.354375\n\n|Package|Version|Last Commit|Stars|Forks|Description|\n|-------|-------|-----------|-----|-----|-----------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|2.4.0|Tue, 05 Jan 2021 07:16:38 GMT|2277|137|[(i)](# "The Python Dict that\'s better than heroin.")|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|6.0.2|Sat, 02 Apr 2022 02:24:21 GMT|2062|89|[(i)](# "Python dictionaries with advanced dot notation access")|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|1.3.30|Wed, 06 Apr 2022 16:26:33 GMT|366|43|[(i)](# "Dot access dictionary with dynamic hierarchy creation and ordered iteration")|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|0.4.0|Wed, 21 Sep 2022 22:23:19 GMT|20|1|[(i)](# "A blazing fast dict subclass that supports dot access notation.")|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|1.9|Sun, 28 Feb 2021 10:26:53 GMT|207|39|[(i)](# "Access dict values as attributes (works recursively)")|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|0.0.3|Sun, 22 Nov 2020 16:48:22 GMT|18|1|[(i)](# "Dot-accessible, update-aware Python dicts (& lists). Works recursively, like a charm.")|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|1.1.5|Thu, 15 Aug 2019 23:10:47 GMT|14|3|[(i)](# "python: a lightweight add-on for dictionaries, featuring deep dictionary union, dictionary keys as object attributes (in code dict.key.sub.value notation) as well as a separate string dig/dug for using strings with dot notation on native dictionaries")|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|0.1.6|Thu, 12 May 2022 20:37:59 GMT|7|2|[(i)](# "Python dictionary with dot notation")|\n\n## Creating an object of the dict subclass\n\nHow the packages create an object of the dict subclass\n\n| |Created|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`{\'a\': {\'b\': {\'c\': 1}}}` **Type**: `Dict`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`{\'a\': {\'b\': {\'c\': 1}}}` **Type**: `Box`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`DotMap(a=DotMap(b=DotMap(c=1)))` **Type**: `DotMap`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`✫(a=✫(b=✫(c=1)))` **Type**: `DotWiz`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`{\'a\': {\'b\': {\'c\': 1}}}` **Type**: `EasyDict`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`{\'a\': {\'b\': {\'c\': 1}}}` **Type**: `DotsiDict`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|`{\'a\': {\'b\': {\'c\': 1}}}` **Type**: `Dict`|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`{\'a\': Diot({\'b\': Diot({\'c\': 1})})}` **Type**: `Diot`|\n\n## Creating a dict with preserved keys\n\nHow the packages create a dict with preserved keys\n(e.g. `keys`, `values`, `items`, etc)\n\nLiterally, `{"keys": 1}`\n\n\n| |Created or error|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|{\'keys\': 1}|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|{\'keys\': 1}|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|DotMap(keys=1)|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|✫(keys=1)|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|{\'keys\': 1}|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|{\'keys\': 1}|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|Key \'keys\' conflicts with reserved word|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|{\'keys\': 1}|\n\n## Creating a dict with magic keys\n\nHow the packages create a dict with magic keys\n(e.g. `__name__`, `__class__`, etc)\n\nLiterally, `{"__name__": 1}`\n\n\n| |Created or error|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`{\'__name__\': 1}`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`{\'__name__\': 1}`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`DotMap(__name__=1)`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`✫(__name__=1)`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`{\'__name__\': 1}`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`{\'__name__\': 1}`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|`{\'__name__\': 1}`|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`{\'__name__\': 1}`|\n\n## Accessing values\n\nHow the packages to access values\n\nLiterally `1` from `{"a": {"b": {"c": 1}}}`\n\n\n| |Way to access value|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|`dictlib.dig(<dict>, \'a.b.c\')` or `dictlib.Dict(<dict>).a.b.c`|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`<dict>.a.b.c` or `<dict>[\'a\'][\'b\'][\'c\']`|\n\n## Recursive Dot Access\n\nWhether recursive dot access is supported when there are lists\nin the dict\n\nLiterally `<dict>.a.b[0].c` from `{"a": {"b": [{"c": 1}, {"d": 2}]}}`\n\n\n| |Value or error|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`1`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`1`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`1`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`1`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`1`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`1`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|AttributeError: \'dict\' object has no attribute \'c\'|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`1`|\n\n## Automatic Hierarchy\n\nWhether a hierarchical structure is created by dot notation\n\nLiterally `<dict>.a.b.c = 1` creates `{"a": {"b": {"c": 1}}}`\n\n\n| |Created or error|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`{\'a\': {\'b\': {\'c\': 1}}}`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|KeyError: "\'Box\' object has no attribute \'a\'"|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`DotMap(a=DotMap(b=DotMap(c=1)))`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|AttributeError: \'DotWiz\' object has no attribute \'a\'|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|AttributeError: \'EasyDict\' object has no attribute \'a\'|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|KeyError: \'a\'|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|KeyError: \'a\'|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|AttributeError: a|\n\n## Accessing values with preserved keys\n\nHow to access values with conflict keys\n\nLiterally, accessing values from `{"keys": 1, "__name__": 2}`\n\n\n|Package|`obj.keys`|`obj[\'keys\']`|`obj.__name__`|`obj[\'__name__\']`|\n|---|---|---|---|---|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`<built-in method keys of Dict object at 0x7f4455a49c70>`|`1`|`2`|`2`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`<bound method Box.keys of Box({\'keys\': 1, \'__name__\': 2})>`|`1`|`2`|`2`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`<bound method DotMap.keys of DotMap(keys=1, __name__=2)>`|`1`|`AttributeError: __name__`|`2`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`1`|`1`|`2`|`2`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`1`|`1`|`2`|`2`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`<built-in method keys of DotsiDict object at 0x7f4455a49c70>`|`1`|`2`|`2`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|`Can\'t create`|`Can\'t create`|`Can\'t create`|`Can\'t create`|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`<built-in method keys of Diot object at 0x7f4455a49c70>`|`1`|`2`|`2`|\n\n## Accessing dashed keys\n\nHow the values with keys with dash are accessed\n\nLiterally `<dict>.a_b` for `{"a-b": 1}`\n\n\n|Package|`obj.a_b`|`obj[\'a_b\']`|`obj[\'a-b\']`|\n|---|---|---|---|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`{}`|`{}`|`1`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`1`|`KeyError: "\'a_b\'"`|`1`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|`DotMap()`|`DotMap()`|`1`|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`AttributeError: \'DotWiz\' object has no attribute \'a_b\'`|`KeyError: \'a_b\'`|`1`|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|`AttributeError: \'EasyDict\' object has no attribute \'a_b\'`|`KeyError: \'a_b\'`|`1`|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|`KeyError: \'a_b\'`|`KeyError: \'a_b\'`|`1`|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|`1`|`1`|`1`|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`1`|`1`|`1`|\n\n## Frozen Dict Support\n\nWhether the packages support frozen dicts\n\n| |Support? and how?|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|`<dict>.freeze()/.unfreeze()`|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|`Box(<dict>, frozen_box=True)`|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|Not supported|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|Not supported|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|Not supported|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|Not supported|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|Not supported|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|`FrozenDiot(<dict>)` or `Diot(<dict>, diot_frozen=True)`|\n\n## Key transformation support\n\nWhether the packages support key transformation for dot access\n\nFor example: making `<dict>.a_b` to access value from `{"a.b": 1}`\n\n\n| |Support? and how?|\n|-|-----------------------|\n|<a target="_blank" href="https://github.com/mewwts/addict">addict</a>|Not supported|\n|<a target="_blank" href="https://github.com/cdgriffith/Box">python-box</a>|Using Conversion Box or Camel Killer Box|\n|<a target="_blank" href="https://github.com/drgrib/dotmap">dotmap</a>|Not supported|\n|<a target="_blank" href="https://github.com/rnag/dotwiz">dotwiz</a>|`DotWizPlus` turns special-cased keys, such as names with spaces, into valid snake_case words|\n|<a target="_blank" href="https://github.com/makinacorpus/easydict">easydict</a>|Not supported|\n|<a target="_blank" href="https://github.com/polydojo/dotsi">dotsi</a>|Not supported|\n|<a target="_blank" href="https://github.com/srevenant/dictlib">dictlib</a>|Not supported|\n|<a target="_blank" href="https://github.com/pwwang/diot">diot</a>|Support custom transform function: `Diot(..., diot_transform =...)`|\n\n',
    'author': 'pwwang',
    'author_email': 'pwwang@pwwang.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
